@@ -25,6 +25,11 @@ class GeoEncoder(BaseEncoder):
         self.index: pd.Index = pd.Index(self.encoder["index"])
 
     def read_source(self) -> Tuple[pd.DataFrame, Dict]:
+        """Parse the location information from `geojson-counties-fips.json`.
+
+        Returns:
+            Tuple[pd.DataFrame, Dict]: dataframe and county-wise dictionary.
+        """
         geo_county_dict: Dict[str, List] = {}
         for item in self.encode_source["features"]:
             coordinates = item["geometry"]["coordinates"]
@@ -42,10 +47,26 @@ class GeoEncoder(BaseEncoder):
         return (dataframe, geo_county_dict)
 
     def encoding_single(self, org: Union[List, str]) -> Union[int, str]:
+        """Encoding for a single location.
+
+        Args:
+            org (Union[List, str]): location to be encoded.
+
+        Returns:
+            Union[int, str]: the coding corresponds to the given target.
+        """
         tree = spatial.KDTree(self.coord)
         return str(self.index[int(tree.query(org)[1])])
 
     def encoding_series(self, series: pd.Series) -> pd.Series:
+        """Encoding for the series of locations.
+
+        Args:
+            series (pd.Series): ths series of locations.
+
+        Returns:
+            pd.Series: the corresponding series of locations.
+        """
         encodings = pd.Series(series.apply(lambda x: self.encoding_single(x)))
         return encodings
 
@@ -55,6 +76,13 @@ class GeoEncoder(BaseEncoder):
         source_name: str,
         hover_name: Optional[str] = None,
     ) -> None:
+        """Method to indicate the values in a geography view.
+
+        Args:
+            dataframe (pd.DataFrame): data source.
+            source_name (str): the name of column for target data.
+            hover_name (Optional[str], optional): the name of column for hovering labels. Defaults to None.
+        """
         max_value: float = np.ceil(dataframe[source_name].max())
         min_value: float = np.floor(dataframe[source_name].min())
         fig = px.choropleth(
