@@ -1,5 +1,5 @@
 from abc import ABCMeta, abstractmethod
-from typing import Dict, Optional, Tuple, TypeVar, Union
+from typing import Dict, List, Optional, Tuple, TypeVar, Union
 
 import numpy as np
 import pandas as pd
@@ -37,9 +37,9 @@ class BaseRecommender(metaclass=ABCMeta):
         random_split: bool,
         user_features: Optional[pd.DataFrame] = None,
         item_features: Optional[pd.DataFrame] = None,
-        toppop_mask: Optional[np.ndarray] = None,
+        toppop_keep: Optional[np.ndarray] = None,
     ) -> None:
-        self.df_interaction, self.userDict, self.itemDict = self.get_interaction(
+        self.df_interaction, self.user_list, self.item_list = self.get_interaction(
             df_interaction
         )
         self.n_users, self.n_items = self.df_interaction.max()[:2] + 1
@@ -49,28 +49,28 @@ class BaseRecommender(metaclass=ABCMeta):
         self.test_size = test_size
         self.random_split = random_split
         self.train_mat, self.test_mat = self.process_interaction()
-        self.toppop: np.ndarray = self._get_toppop(toppop_mask)
+        self.toppop: np.ndarray = self._get_toppop_keep(toppop_keep)
 
     def get_interaction(
         self, df_interaction: pd.DataFrame
-    ) -> Tuple[pd.DataFrame, Dict, Dict]:
+    ) -> Tuple[pd.DataFrame, List, List]:
         """The converter for input dataframe
 
         Args:
             df_interaction(pd.DataFrame): user/item interaction matrix.
-                columns should be ["userId", "itemId", "interaction"]
+                columns should be ["userId", "itemId"]
 
         """
-
-        dataframe = df_interaction.iloc[:, :3]
-        dataframe.columns = pd.Index(["userId", "itemId", "interaction"])
+        dataframe = df_interaction.iloc[:, :2]
+        dataframe.columns = pd.Index(["userId", "itemId"])
+        dataframe["itemId"] = dataframe["itemId"].apply(str)
         user_cat = pd.Categorical(dataframe["userId"])
         item_cat = pd.Categorical(dataframe["itemId"])
         dataframe["userId"] = user_cat.codes
         dataframe["itemId"] = item_cat.codes
-        userDict = dict(enumerate(user_cat.categories))
-        itemDict = dict(enumerate(item_cat.categories))
-        return dataframe, userDict, itemDict
+        user_list = list(user_cat.categories)
+        item_list = list(item_cat.categories)
+        return dataframe, user_list, item_list
 
     def process_interaction(self) -> Tuple[sps.coo_matrix, Optional[sps.coo_matrix]]:
         dataframe = self.df_interaction
@@ -79,13 +79,14 @@ class BaseRecommender(metaclass=ABCMeta):
             dataframe = dataframe.sample(frac=1, random_state=42).reset_index(drop=True)
 
         self.df_interaction = dataframe
+        dataframe["interaction"] = 1
 
         if self.test_size != 0:
             train, test = train_test_split(
                 dataframe, test_size=self.test_size, shuffle=False
             )
             test_mat = sps.coo_matrix(
-                (test.interaction, (test.userId, test.itemId)),
+                (np.ones(len(test)), (test.userId, test.itemId)),
                 (self.n_users, self.n_items),
                 "int32",
             )
@@ -93,14 +94,14 @@ class BaseRecommender(metaclass=ABCMeta):
             train = dataframe
             test_mat = None
         train_mat = sps.coo_matrix(
-            (train.interaction, (train.userId, train.itemId)),
+            (np.ones(len(train)), (train.userId, train.itemId)),
             (self.n_users, self.n_items),
             "int32",
         )
         return train_mat, test_mat
 
     def clean_items(self) -> pd.DataFrame:
-        invmap = {v: k for k, v in self.itemDict.items()}
+        invmap = {v: k for k, v in enumerate(self.item_list)}
         self.items["encodes"] = self.items["itemId"].apply(
             lambda x: self._encode(x, invmap)
         )
@@ -133,13 +134,25 @@ class BaseRecommender(metaclass=ABCMeta):
     ) -> np.ndarray:
         raise NotImplementedError("predict must be implemented.")
 
-    def _get_toppop(self, toppop_mask: Optional[np.ndarray]) -> np.ndarray:
+    def _get_toppop_keep(self, toppop_keep: Optional[np.ndarray]) -> np.ndarray:
+        """Get the top popular indices for items.
+
+        Args:
+            toppop_keep (Optional[np.ndarray]): The indices of items to be kept.
+
+        Returns:
+            np.ndarray:
+                Top popular indices of items.
+                Return all top popular if `toppop_keep` is not assigned.
+        """
         scores = np.asarray(self.train_mat.sum(axis=0)).ravel()
-        if toppop_mask:
-            mask = np.zeros(scores.shape[0], dtype=bool)
-            mask[toppop_mask] = True
+        if toppop_keep is not None:
+            mask = np.ones(scores.shape[0], dtype=bool)
+            mask[toppop_keep] = False
             scores[mask] = 0
-        return (-scores).argsort().argsort()
+            return (-scores).argsort()[: len(toppop_keep)]
+        else:
+            return (-scores).argsort()
 
     def predict_for_userId(self, user_id: int) -> np.ndarray:
         user_ids: np.ndarray = np.full(self.n_items, user_id)
@@ -169,3 +182,51 @@ class BaseRecommender(metaclass=ABCMeta):
             by="scores", ascending=False, ignore_index=True
         )
         return candidates.merge(self.items, how="left", on="itemId")
+
+    def get_user_id(self, user_string: str) -> Optional[int]:
+        """Get the `user_id` for a given `user_string`.
+
+        Args:
+            user_string (str): Original user string.
+
+        Returns:
+            Optional[int]:
+                The index of user_id in `user_list`.
+                Return `None` it not in training set.
+        """
+        try:
+            user_id = self.user_list.index(user_string)
+        except:
+            user_id = None
+        return user_id
+
+    def get_item_id(self, item_string: str) -> Optional[int]:
+        """Get the `item_id` for a given `item_string`.
+
+        Args:
+            item_string (str): Original item string.
+
+        Returns:
+            Optional[int]:
+                The index of item_id in `item_list`.
+                Return `None` it not in training set.
+        """
+        try:
+            item_id = self.item_list.index(item_string)
+        except:
+            item_id = None
+        return item_id
+
+    def get_topk_indices(self, scores: np.ndarray, top_k: int) -> np.ndarray:
+        """Get the indices correspond to the topk items.
+
+        Args:
+            scores (np.ndarray): Scores given by the models.
+            top_k (int): Numbers of top items to be kept.
+
+        Returns:
+            np.ndarray: Indices for `top_k` items.
+        """
+        indices = np.argpartition(scores, -top_k)[-top_k:]
+        sorted_indices = indices[np.argsort(scores[indices])[::-1]]
+        return np.asarray(sorted_indices)
