@@ -10,6 +10,16 @@ from .base import BaseRecommender
 
 
 class IALSRecommender(BaseRecommender):
+    """iALS recommender based on `implicit`.
+
+    Args:
+        interaction (pd.DataFrame): user-item interaction matrix.
+        items (Optional[pd.DataFrame]): item side information.
+        factors (int): the dimensions of user/item embeddings.
+        regularization (float): regularization coefficient.
+        alpha (float): the unobserved weight.
+    """
+
     def __init__(
         self,
         df_interaction: pd.DataFrame,
@@ -58,13 +68,23 @@ class IALSRecommender(BaseRecommender):
         id_list: List = [list(id) for id in ids]
         return (id_list, scores)
 
-    def recommend_single(self, user_token: str, top_k: int = 100) -> np.ndarray:
-        if user_token in self.userDict:
-            user_id = self.userDict[user_token]
+    def recommend_single(self, user_string: str, top_k: int = 100) -> List:
+        """Recommend for single user with `top_k` items.
+
+        Args:
+            user_string (str): the original token string for user.
+            top_k (int, optional): `top_k` items to be recommended. Defaults to 100.
+
+        Returns:
+            List: a list of recommended item ids.
+        """
+        if user_string in self.user_list:
+            user_id = self.get_user_id(user_string)
             ids, _ = self.ials.recommend(user_id, self.train_mat[user_id], N=top_k)
-            return np.asarray(ids)
+            indice = np.asarray(ids)
         else:
-            return self.toppop
+            indice = self.toppop[:top_k]
+        return [self.item_list[index] for index in indice]
 
     def auc_score(self, top_k: int = 100) -> float:
         return float(AUC_at_k(self.ials, self.train_mat, self.test_mat, K=top_k))
@@ -83,6 +103,57 @@ class IALSRecommender(BaseRecommender):
         mask[keep_row] = False
         self.ials.item_factors[mask] = 0
 
+    def get_score_single_user(
+        self, user_string: str, keep_indices: np.ndarray
+    ) -> Optional[np.ndarray]:
+        """Get the single user's predictions scores for the filtered items.
+        Return `None` for new users.
+
+        Args:
+            user_string (str): Original user token string.
+            keep_indices (np.ndarray): Items to be kept based on filters.
+
+        Returns:
+            Optional[np.ndarray]:
+                Predictions for the given items.
+                Return `None` for new users.
+        """
+        user_id = self.get_user_id(user_string)
+        if user_id:
+            user_factor = self.get_user_factors()[user_id]
+            item_factors = self.get_item_factors()[keep_indices]
+            scores = np.asarray(user_factor @ item_factors.T)
+            return scores
+        else:
+            return None
+
+    def get_topk_single_user(
+        self,
+        user_string: str,
+        keep_indices: np.ndarray,
+        top_k: int,
+    ) -> np.ndarray:
+        """Get the recommended item ids for a given user id.
+
+        Args:
+            user_string (str): User id string.
+            keep_indices (np.ndarray): Indices for items to be kept.
+            top_k (int): Top-k items to be recommended.
+
+        Returns:
+            np.ndarray:
+                Recommended items ids.
+                Return `toppop` for new users.
+        """
+        scores = self.get_score_single_user(user_string, keep_indices)
+        if scores is None:
+            original_indices = self._get_toppop_keep(keep_indices)[:top_k]
+        else:
+            rank = self.get_topk_indices(scores, top_k)
+            original_indices = keep_indices[rank]
+        topk_items = np.asarray([self.item_list[index] for index in original_indices])
+        return topk_items
+
     def predict(
         self,
         user_ids: np.ndarray,
